/*********************************************************************************
 *																				
 *   e-Mail Notify - Internet e-Mail Notifier
 *   Copyright (C) 1996-1998 Ludovic Dubost <ldubost@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 **********************************************************************************/



#include "stdafx.h"
#include "email.h"
#include "AppBarWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define ID_TIMER_SCROLL 200  



// AppBarQuerySetPos - sets the size and position of an appbar. 
// uEdge - screen edge to which the appbar is to be anchored 
// lprc - current bounding rectangle of the appbar 
// pabd - address of APPBARDATA structure with the hWnd and 
//     cbSize members filled 
void PASCAL AppBarQuerySetPos(UINT uEdge, LPRECT lprc, PAPPBARDATA pabd) 
{ 
    int iHeight = 0; 
    int iWidth = 0; 
 
    pabd->rc = *lprc; 
    pabd->uEdge = uEdge; 
 
    // Copy the screen coordinates of the appbar's bounding 
    // rectangle into the APPBARDATA structure. 
    if ((uEdge == ABE_LEFT) || 
            (uEdge == ABE_RIGHT)) { 
        iWidth = pabd->rc.right - pabd->rc.left; 
        pabd->rc.top = 0; 
        pabd->rc.bottom = GetSystemMetrics(SM_CYSCREEN); 
    } else { 
        iHeight = pabd->rc.bottom - pabd->rc.top; 
        pabd->rc.left = 0; 
        pabd->rc.right = GetSystemMetrics(SM_CXSCREEN); 
    } 
 
    // Query the system for an approved size and position. 
    SHAppBarMessage(ABM_QUERYPOS, pabd); 
 
    // Adjust the rectangle, depending on the edge to which the 
    // appbar is anchored. 
    switch (uEdge) { 
        case ABE_LEFT: 
            pabd->rc.right = pabd->rc.left + iWidth; 
            break; 
 
        case ABE_RIGHT: 
            pabd->rc.left = pabd->rc.right - iWidth; 
            break; 
 
        case ABE_TOP: 
            pabd->rc.bottom = pabd->rc.top + iHeight; 
            break; 
 
        case ABE_BOTTOM: 
            pabd->rc.top = pabd->rc.bottom - iHeight; 
            break; 
    } 
 
    // Pass the final bounding rectangle to the system. 
    SHAppBarMessage(ABM_SETPOS, pabd); 
 
    // Move and size the appbar so that it conforms to the 
    // bounding rectangle passed to the system. 
    MoveWindow(pabd->hWnd, pabd->rc.left, pabd->rc.top, 
        pabd->rc.right - pabd->rc.left, 
        pabd->rc.bottom - pabd->rc.top, TRUE); 
} 
 
  
  
// AppBarPosChanged - adjusts the appbar's size and position. 
// pabd - address of an APPBARDATA structure that contains information 
//     used to adjust the size and position 
void PASCAL AppBarPosChanged(UINT uEdge, PAPPBARDATA pabd) 
{ 
    RECT rc; 
    RECT rcWindow; 
    int iHeight; 
    int iWidth; 
 
    rc.top = 0; 
    rc.left = 0; 
    rc.right = GetSystemMetrics(SM_CXSCREEN); 
    rc.bottom = GetSystemMetrics(SM_CYSCREEN); 
 
    GetWindowRect(pabd->hWnd, &rcWindow); 
    iHeight = rcWindow.bottom - rcWindow.top; 
    iWidth = rcWindow.right - rcWindow.left; 
 
    switch (uEdge) { 
        case ABE_TOP: 
            rc.bottom = rc.top + iHeight; 
            break; 
 
        case ABE_BOTTOM: 
            rc.top = rc.bottom - iHeight; 
            break; 
 
        case ABE_LEFT: 
            rc.right = rc.left + iWidth; 
            break; 
 
        case ABE_RIGHT: 
            rc.left = rc.right - iWidth; 
            break; 
        } 
        AppBarQuerySetPos(uEdge, &rc, pabd); 
    } 
 




/////////////////////////////////////////////////////////////////////////////
// CAppBarWnd

IMPLEMENT_DYNCREATE(CAppBarWnd, CWnd)

CAppBarWnd::CAppBarWnd()
{
 m_bAppRegistered = FALSE;   
 m_bStarted = FALSE;
 m_csText = _T("");
 m_uPosition = 0;
 m_uMaxPosition = 100;
 m_bVisible = FALSE;
 m_uSpeed = 10;

 m_cfont.CreateFont(12,12,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,0,0,0,0,"Times New Roman");
}

CAppBarWnd::~CAppBarWnd()
{
}


BEGIN_MESSAGE_MAP(CAppBarWnd, CWnd)
	//{{AFX_MSG_MAP(CAppBarWnd)
	ON_WM_DESTROY()
	ON_WM_ACTIVATE()
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_APPBAR_STOP, OnAppbarStop)
	//}}AFX_MSG_MAP
	// ON_MESSAGE(WM_WINDOWPOSCHANGED,OnActivate)
	ON_MESSAGE(APPBAR_CALLBACK,AppBarCallback)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAppBarWnd message handlers


// RegisterAccessBar - registers or unregisters an appbar. 
// Returns TRUE if successful or FALSE otherwise. 
// hwndAccessBar - handle of the appbar 
// fRegister - register and unregister flag 
// 
// Global variables 
//     g_uSide - screen edge (defaults to ABE_TOP) 
//     g_fAppRegistered - flag indicating whether the bar is registered 
BOOL CAppBarWnd::RegisterAccessBar(BOOL bRegister) 
{ 
    // Specify the structure size and handle of the appbar. 
    m_AppBar.cbSize = sizeof(APPBARDATA); 
    m_AppBar.hWnd = GetSafeHwnd(); 
 
    if (bRegister && !m_bAppRegistered) 
	{ 
         // Provide an identifier for notification messages. 
        m_AppBar.uCallbackMessage = APPBAR_CALLBACK; 
 
        // Register the appbar. 
        if (!SHAppBarMessage(ABM_NEW, &m_AppBar)) 
            return FALSE; 
        
   	    // Set the default size and position of the access bar.
        AppBarPosChanged(ABE_TOP, &m_AppBar);
		ShowWindow(SW_SHOW);
		m_bAppRegistered = TRUE; 
	} 
    
	if (!bRegister && m_bAppRegistered) 
	{ 
		StartScroll(FALSE);
		// Unregister the appbar. 
        SHAppBarMessage(ABM_REMOVE, &m_AppBar); 
        m_bAppRegistered = FALSE; 
    } 
    return TRUE; 
} 
 
 

 
  

// AppBarCallback - processes notification messages sent by the system. 
// hwndAccessBar - handle of the appbar 
// uNotifyMsg - identifier of the notification message 
// lParam - message parameter 
LONG CAppBarWnd::AppBarCallback(UINT uNotifyMsg,LPARAM lParam) 
{ 
    APPBARDATA abd; 
    UINT uState; 
 
    abd.cbSize = sizeof(abd); 
    abd.hWnd = GetSafeHwnd(); 
 
    switch (uNotifyMsg) { 
        case ABN_STATECHANGE: 
 
            // Check to see if the taskbar's always-on-top state has 
            // changed and, if it has, change the appbar's state 
            // accordingly. 
            uState = SHAppBarMessage(ABM_GETSTATE, &abd); 
            ::SetWindowPos(GetSafeHwnd(), 
                (ABS_ALWAYSONTOP & uState) ? HWND_TOPMOST : HWND_BOTTOM, 
                0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE); 
            break; 
 
        case ABN_FULLSCREENAPP: 
 
            // A full screen application has started, or the last full 
            // screen application has closed. Set the appbar's 
            // Z order appropriately. 
            if (lParam) { 
                ::SetWindowPos(GetSafeHwnd(), 
                    (ABS_ALWAYSONTOP & uState) ? 
                                            HWND_TOPMOST : HWND_BOTTOM, 
                    0, 0, 0, 0, 
                    SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE); 
            } else { 
                uState = SHAppBarMessage(ABM_GETSTATE, &abd); 
                if (uState & ABS_ALWAYSONTOP) 
                    ::SetWindowPos(GetSafeHwnd(), HWND_TOPMOST, 
                        0, 0, 0, 0, 
                        SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE); 
        } 
 
        case ABN_POSCHANGED: 
 
            // The taskbar or another appbar has changed its 
            // size or position. 
            AppBarPosChanged(m_AppBar.uEdge,&abd); 
            break; 
    } 
 return 0;
} 
 
   


void CAppBarWnd::OnDestroy() 
{
 if (m_bAppRegistered)
 	RegisterAccessBar(FALSE);   // Unregister the access bar.
 
 CWnd::OnDestroy();
}

void CAppBarWnd::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CWnd::OnActivate(nState, pWndOther, bMinimized);
    APPBARDATA abd;
    abd.cbSize = sizeof(APPBARDATA);
    abd.hWnd = GetSafeHwnd();
    abd.lParam = (LONG)NULL;
    SHAppBarMessage(ABM_ACTIVATE, &abd);
}


BOOL CAppBarWnd::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	return CWnd::OnCommand(wParam, lParam);
}

void CAppBarWnd::Start()
{
  if (!m_bStarted)
  {
   const char* AppBarClass = AfxRegisterWndClass(0,LoadCursor(NULL,IDC_ARROW),GetSysColorBrush(COLOR_BTNFACE),0);
   CreateEx(WS_EX_TOOLWINDOW , 
		    AppBarClass, "AppBar",
			WS_POPUP | WS_CLIPCHILDREN ,
			0,0,DEF_APPBAR_WIDTH, DEF_APPBAR_HEIGHT,
			NULL, NULL, NULL);
   m_bStarted = TRUE;
  }
  else
  {
   RegisterAccessBar(FALSE);
   DestroyWindow();	
   m_bStarted = FALSE;	  
  }
}

void CAppBarWnd::SetText(const CString& csText)
{
 m_csText = "";
 for (unsigned i=0;i<csText.GetLength();i++)
	{
	 char c=csText.GetAt(i);
	 if (isprint(c))
		 m_csText += c;
    }
 
 UpdateScrollRange();
 UpdateScrollBar();
}

void CAppBarWnd::AddText(const CString& csText)
{
 for (unsigned i=0;i<csText.GetLength();i++)
	{
	 char c=csText.GetAt(i);
	 if (isprint(c))
		 m_csText += c;
    }
 UpdateScrollRange();
 UpdateScrollBar();
}

void CAppBarWnd::SetPosition(UINT uPos)
{
 m_uPosition = uPos;
 UpdateScrollBar();
}

void CAppBarWnd::ShowText(UINT uStyle)
{
 m_bVisible = uStyle;
 UpdateScrollBar();
}


void CAppBarWnd::StartScroll(BOOL bStart)
{
 if (m_bScrolling)
 {
  if (!bStart)
   {
   KillTimer(ID_TIMER_SCROLL);
   m_bScrolling = FALSE;
   }
 }
 else
 {
  if (bStart)
   {
    SetTimer(ID_TIMER_SCROLL,100,NULL);
    m_bScrolling = TRUE;
  }
 }
}


void CAppBarWnd::SetSpeed(UINT uSpeed)
{
 m_uSpeed = uSpeed;
}

void CAppBarWnd::OnPaint() 
{
 CPaintDC dc(this); // device context for painting
	
  dc.ExcludeClipRect(0,0,50,DEF_APPBAR_WIDTH);
  dc.SetBkColor(GetSysColor(COLOR_BTNFACE));
  UINT uWidth = GetSystemMetrics(SM_CXSCREEN);
  long uPos = uWidth - (uWidth*m_uPosition)/100;

 if (m_bVisible && m_csText.GetLength())
 {
  dc.ExtTextOut(uPos,4,0,NULL,m_csText+"   ",NULL);
 }
 else
 {
  dc.ExtTextOut(uPos,4,ETO_OPAQUE,NULL,m_csText+"   ",NULL);
 }
	
	// Do not call CWnd::OnPaint() for painting messages
}

void CAppBarWnd::UpdateScrollBar()
{
 Invalidate(FALSE);
}

void CAppBarWnd::OnTimer(UINT nIDEvent) 
{
	m_uPosition++;
	if (m_uPosition>=m_uMaxPosition)
		m_uPosition = 0;
	UpdateScrollBar();
	
	CWnd::OnTimer(nIDEvent);
}


void CAppBarWnd::UpdateScrollRange()
{
 CDC* dc = GetDC();
 UINT uWidth = GetSystemMetrics(SM_CXSCREEN);
 
 m_uMaxPosition = 100*(dc->GetTextExtent(m_csText).cx + uWidth)/uWidth;
 ReleaseDC(dc);
}

void CAppBarWnd::OnRButtonUp(UINT nFlags, CPoint point) 
{
 CMenu popup;
		 										
#ifndef FREEWARE
 if (!popup.LoadMenu(IDR_POPUP_MENU))
   AfxThrowResourceException();	
#else
 if (!popup.LoadMenu(IDR_POPUP_MENU_FW))
   AfxThrowResourceException();	
#endif
 
 CMenu* pMenu = popup.GetSubMenu(1);
 pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_RIGHTBUTTON,
                       point.x,point.y,this,NULL);
 popup.DestroyMenu();
 CWnd::OnRButtonUp(nFlags, point);
}

void CAppBarWnd::OnAppbarStop() 
{
}
