/*********************************************************************************
 *																				
 *   e-Mail Notify - Internet e-Mail Notifier
 *   Copyright (C) 1996-1998 Ludovic Dubost <ldubost@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 **********************************************************************************/
 
 //
 // Windows 3.1 View Implementation
 //

#include "stdafx.h"
#include "email.h"
#include "emaildoc.h"
#include "emailvw.h"
#include <stdlib.h>
#include <limits.h> 


IMPLEMENT_DYNCREATE(CEmailView, CScrollView)

BEGIN_MESSAGE_MAP(CEmailView,CScrollView)
//{{AFX_MSG_MAP(CEmailView)
	ON_WM_KEYDOWN()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


CEmailView::CEmailView()
{
 m_nPrevSelectedRow = 0;
}

void CEmailView::OnInitialUpdate()
{
	m_nPrevRowCount = GetRowCount();
	m_nPrevSelectedRow = GetActiveRow();
}

void CEmailView::OnUpdate(CView*, LPARAM lHint, CObject* pHint)
{
	if (pHint != NULL && pHint->IsKindOf(RUNTIME_CLASS(CEmailHint)))
	{
		int nRow = LOWORD(lHint);
		UpdateRow(nRow);
	}
	else
	{
		Invalidate();
	}
}

void CEmailView::GetRowWidthHeight(CDC* pDC, int& nRowWidth, int& nRowHeight)
{
	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);
	nRowWidth = tm.tmAveCharWidth * ROW_WIDTH;
	nRowHeight = tm.tmHeight * 2; 
}

int CEmailView::GetActiveRow()
{
  return (GetDocument()->GetMailSelected()-1);
}

int CEmailView::GetRowCount()
{
  return (GetDocument()->GetMailNb());
}

void CEmailView::ChangeSelectionNextRow(BOOL bNext)
{
 	GetDocument()->ChangeSelectionNext(bNext);
}

void CEmailView::ChangeSelectionToRow(int nRow)
{
 	GetDocument()->ChangeSelectionTo(nRow);
}

void CEmailView::OnDrawRow(CDC* pDC, int nRow, int y, BOOL bSelected)
{
	CBrush brushBackground;

	COLORREF crOldText = 0;
	COLORREF crOldBackground = 0;

	if (bSelected)
	 {
		brushBackground.CreateSolidBrush(::GetSysColor(COLOR_HIGHLIGHT));
		crOldBackground = pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
		crOldText = pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
	 }
	else
	 {
		brushBackground.CreateSolidBrush(::GetSysColor(COLOR_WINDOW));
		crOldBackground = pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));
	 }

	CRect rectSelection;
	pDC->GetClipBox(&rectSelection);
	rectSelection.top = y;
	rectSelection.bottom = y + m_nRowHeight;
	pDC->FillRect(&rectSelection, &brushBackground);


	CString strAuthor, strSubject, strDate;
	unsigned long ulSize;

	GetDocument()->GetMail(nRow, strAuthor, strSubject, strDate, ulSize);

	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);

	pDC->TextOut(AUTHOR_COL*tm.tmAveCharWidth, y, strAuthor.Left(AUTHOR_LEN));
	pDC->TextOut(DATE_COL*tm.tmAveCharWidth, y, strDate.Left(DATE_LEN));

	pDC->TextOut(SUBJECT_COL*tm.tmAveCharWidth, y+tm.tmHeight, strSubject.Left(SUBJECT_LEN));

	char sbuffer[100];
	sprintf(sbuffer,"%lu bytes",ulSize);
	CString strSize = sbuffer;

	pDC->TextOut(SIZE_COL*tm.tmAveCharWidth, y+tm.tmHeight, strSize.Left(SIZE_LEN));
	
	if (bSelected)
	{
		pDC->SetBkColor(crOldBackground);
		pDC->SetTextColor(crOldText);
	}
}

void CEmailView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	CClientDC dc(this);
	OnPrepareDC(&dc);
	dc.DPtoLP(&point);
	CRect rect(point, CSize(1,1));
	int nFirstRow, nLastRow;
	RectLPtoRowRange(rect, nFirstRow, nLastRow, TRUE);
	if (nFirstRow <= (GetRowCount() - 1))
		{
		 ChangeSelectionToRow(nFirstRow);
	     
		 if (GetDocument()->Downloading()==0)
		  {
	       POINT pCursor;
		   CMenu popup;
		 	
		   GetCursorPos(&pCursor);
		 										
#ifndef FREEWARE
 if (!popup.LoadMenu(IDR_POPUP_MENU))
   AfxThrowResourceException();	
#else
 if (!popup.LoadMenu(IDR_POPUP_MENU_FW))
   AfxThrowResourceException();	
#endif
 
		   CMenu* pMenu = popup.GetSubMenu(1);
		   pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_RIGHTBUTTON,
		                        pCursor.x,pCursor.y,this,NULL);
	       popup.DestroyMenu();
          } 	    
	    }
    else
   	 CEmailView::OnRButtonUp(nFlags, point);
}


void CEmailView::UpdateRow(int nInvalidRow)
{
	int nRowCount = GetRowCount();
	if (nRowCount != m_nPrevRowCount)
	{
		UpdateScrollSizes();
		m_nPrevRowCount = nRowCount;
	}

	CClientDC dc(this);
	OnPrepareDC(&dc);

	int nFirstRow, nLastRow;
	CRect rectClient;
	GetClientRect(&rectClient);
	dc.DPtoLP(&rectClient);
	RectLPtoRowRange(rectClient, nFirstRow, nLastRow, FALSE);

	POINT pt;
	pt.x = 0;
	BOOL bNeedToScroll = TRUE;
	if (nInvalidRow < nFirstRow)
	{
		pt.y = RowToYPos(nInvalidRow);
	}
	else if (nInvalidRow > nLastRow)
	{
		pt.y = max(0, RowToYPos(nInvalidRow+1) - rectClient.Height());
	}
	else
	{
		bNeedToScroll = FALSE;
	}
	if (bNeedToScroll)
	{
		ScrollToDevicePosition(pt);

		OnPrepareDC(&dc);  
	}

	CRect rectInvalid = RowToWndRect(&dc, nInvalidRow);
	InvalidateRect(&rectInvalid);

	int nSelectedRow = GetActiveRow();
	if (m_nPrevSelectedRow != nSelectedRow)
	{
		CRect rectOldSelection = RowToWndRect(&dc, m_nPrevSelectedRow);
		InvalidateRect(&rectOldSelection);
		m_nPrevSelectedRow = nSelectedRow;
	}


}

void CEmailView::UpdateScrollSizes()
{
	CRect rectClient;
	GetClientRect(&rectClient);

	CClientDC dc(this);
	CalculateRowMetrics(&dc);

	CSize sizeTotal(m_nRowWidth,
		m_nRowHeight * (min(GetRowCount(), LastViewableRow())));

	CSize sizePage(m_nRowWidth/5,
			max(m_nRowHeight,
				((rectClient.bottom/m_nRowHeight)-1)*m_nRowHeight));

	CSize sizeLine(m_nRowWidth/20, m_nRowHeight);

	SetScrollSizes(MM_TEXT, sizeTotal, sizePage, sizeLine);
}


void CEmailView::OnDraw(CDC* pDC)
{
	if (GetRowCount() == 0)
		return;

	
	int nFirstRow, nLastRow;
	CRect rectClip;
	pDC->GetClipBox(&rectClip); 
	RectLPtoRowRange(rectClip, nFirstRow, nLastRow, TRUE);

	int nActiveRow = GetActiveRow();
	int nRow, y;
	int nLastViewableRow = LastViewableRow();
	for (nRow = nFirstRow,
			y = m_nRowHeight * nFirstRow;
			nRow <= nLastRow;
			nRow++, y += m_nRowHeight)
	{
		if (nRow > nLastViewableRow)
		{
			pDC->TextOut(0, y, "Too many rows");
			break;
		}
		OnDrawRow(pDC, nRow, y, nRow == nActiveRow);
	}
}

void CEmailView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	CScrollView::OnPrepareDC(pDC, pInfo);
	CalculateRowMetrics(pDC);
}



int CEmailView::RowToYPos(int nRow)
{
	return (nRow * m_nRowHeight);
}

CRect CEmailView::RowToWndRect(CDC* pDC, int nRow)
{
	int nHorzRes = pDC->GetDeviceCaps(HORZRES);;
	CRect rect(0, nRow * m_nRowHeight,
		nHorzRes, (nRow + 1) * m_nRowHeight);
	pDC->LPtoDP(&rect);
	return rect;
}


int CEmailView::LastViewableRow()
{
	return (INT_MAX / m_nRowHeight - 1);
}

void CEmailView::RectLPtoRowRange(const CRect& rect,
			int& nFirstRow, int& nLastRow,
			BOOL bIncludePartiallyShownRows)
{
	int nRounding = bIncludePartiallyShownRows? 0 : (m_nRowHeight - 1);
	nFirstRow = (rect.top + nRounding) / m_nRowHeight;
	nLastRow = min( (rect.bottom - nRounding) / m_nRowHeight,
		GetRowCount() - 1);
}

void CEmailView::OnSize(UINT nType, int cx, int cy)
{
	UpdateScrollSizes();
	CScrollView::OnSize(nType, cx, cy);
}

void CEmailView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch (nChar)
	{
		case VK_HOME:
			ChangeSelectionToRow(0);
			break;
		case VK_END:
			ChangeSelectionToRow(GetRowCount() - 1);
			break;
		case VK_UP:
			ChangeSelectionNextRow(FALSE);
			break;
		case VK_DOWN:
			ChangeSelectionNextRow(TRUE);
			break;
		case VK_PRIOR:
			OnVScroll(SB_PAGEUP,
				0,  
				GetScrollBarCtrl(SB_VERT));
			break;
		case VK_NEXT:
			OnVScroll(SB_PAGEDOWN,
				0,  
				GetScrollBarCtrl(SB_VERT));
			break;
		default:
			CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
}

void CEmailView::OnLButtonDown(UINT, CPoint point)
{
	CClientDC dc(this);
	OnPrepareDC(&dc);
	dc.DPtoLP(&point);
	CRect rect(point, CSize(1,1));
	int nFirstRow, nLastRow;
	RectLPtoRowRange(rect, nFirstRow, nLastRow, TRUE);
	if (nFirstRow <= (GetRowCount() - 1))
		ChangeSelectionToRow(nFirstRow);
}
