/*********************************************************************************
 *																				
 *   e-Mail Notify - Internet e-Mail Notifier
 *   Copyright (C) 1996-1998 Ludovic Dubost <ldubost@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 **********************************************************************************/

#include "stdafx.h"

#include <afxpriv.h>
#include <mmsystem.h>
#include <stdarg.h>

#include "email.h"
#include "mainfrm.h"
#include "emaildoc.h"
#include "tray.h"
#include "psoption.h"
#include "message.h"
#include "pwdlg.h"

#include "debug.h"

#ifdef LISTVIEW
#include "emlistvw.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
					   


extern BOOL bWin4;
extern BOOL bRasOk;

  
/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_INITMENU()
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_MAIL_CHECKMAIL, OnMailCheckmail)
	ON_WM_TIMER()
	ON_COMMAND(ID_STOP_UPDATE, OnStopUpdate)
	ON_UPDATE_COMMAND_UI(ID_MAIL_CHECKMAIL, OnUpdateMailCheckmail)
	ON_UPDATE_COMMAND_UI(ID_MAIL_OPTIONS, OnUpdateMailOptions)
	ON_UPDATE_COMMAND_UI(ID_STOP_UPDATE, OnUpdateStopUpdate)
	ON_UPDATE_COMMAND_UI(ID_CLEAR_MAIL_WINDOW, OnUpdateClearMailWindow)
	ON_COMMAND(ID_CLEAR_MAIL_WINDOW, OnClearMailWindow)
	ON_WM_CLOSE()
	ON_WM_DESTROY()
	ON_COMMAND(ID_SHOW_EMAIL, OnShowEmail)
	ON_COMMAND(ID_AUTO_TRANSFER, OnAutoTransfer)
	ON_UPDATE_COMMAND_UI(ID_APP_ABOUT, OnUpdateAppAbout)
	ON_COMMAND(ID_MAIL_OPTIONS, OnMailOptions2)
	ON_COMMAND(ID_MAIL_READER, OnMailReader)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_QUERYDRAGICON()
	ON_COMMAND(ID_APP_EXIT, OnAppExit)
	ON_COMMAND(ID_HELP_INDEX, OnHelpIndex)
	ON_UPDATE_COMMAND_UI(ID_MAIL_DELETE, OnUpdateMailFunction)
	ON_COMMAND(ID_HELP, OnHelp)
	ON_WM_SIZE()
	ON_WM_MOVE()
	ON_COMMAND(ID_MAIL_DELETE, OnMailDelete)
	ON_UPDATE_COMMAND_UI(ID_MAIL_VIEW, OnUpdateMailFunction)
	ON_COMMAND(ID_MAIL_VIEW, OnMailView)
	//}}AFX_MSG_MAP
	// Global help commands
#ifdef NEWTOOLTIP
	ON_COMMAND(ID_TEST_TOOLTIP, OnTestTooltip)
#endif
#ifndef WIN16
	ON_COMMAND(ID_VIEW_STATUS_BAR, OnViewStatusBar)
	ON_COMMAND(ID_VIEW_TOOLBAR, OnViewToolbar)
#endif
	ON_MESSAGE(WM_UPDATE_THREAD_FINISHED,OnUpdateThreadFinished)
	ON_MESSAGE(WM_DELETE_THREAD_FINISHED,OnDeleteThreadFinished)
	ON_MESSAGE(WM_VIEW_THREAD_FINISHED,OnViewThreadFinished)
	ON_MESSAGE(WM_NOTIFY_ICON,OnNotifyIcon)
	ON_MESSAGE(WM_DRAWITEM,OnDrawItem)
	ON_MESSAGE(WM_ASYNC_DNS,OnAsyncDNS)
	ON_MESSAGE(WM_SET_MESSAGE,OnSetMessage)
	ON_MESSAGE(WM_MAIL_DOWNLOAD,OnMailDownload)
	ON_MESSAGE(WM_NOTIFY_PROGRESS,OnNotifyProgress)


	ON_WM_SYSCOMMAND()

//	ON_COMMAND(ID_HELP_USING, CFrameWnd::OnHelpUsing)
//	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
//	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars
	
// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
	ID_CLEAR_MAIL_WINDOW,
		ID_SEPARATOR,
	ID_MAIL_CHECKMAIL,
		ID_SEPARATOR,
	ID_MAIL_DELETE,
	ID_MAIL_VIEW,
	ID_SEPARATOR,
	ID_MAIL_OPTIONS,
		ID_SEPARATOR,
	ID_SHOW_EMAIL,
	ID_AUTO_TRANSFER,
 		ID_SEPARATOR,
	ID_APP_ABOUT
};

static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};



#if defined(WIN95) || defined(WINNT35)
UINT uMsg = WM_RASDIALEVENT;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
 popclient = new CPOPClient(AfxGetEmailApp()->uSettingsNb);	 
 hAsyncDNS = 0;	
 UpdateState = ST_IDLE;
 bInit = FALSE;	
 bAnimate = FALSE;
 bDialUp = FALSE;
 ErrorState = 0;
 IdIcon = IDI_SETA_EMPTY;
 TickCount = 0;
#if defined(WIN95) || defined(WINNT35)
 hrcRASHandle = NULL;
#endif

 m_bAsyncOk = FALSE;
#ifdef NEWTOOLTIP
 notifystatus = NULL;
#endif
}

CMainFrame::~CMainFrame()
{
 popup.DestroyMenu();
 delete popclient;
}

#ifndef WIN32
void CMainFrame::SetMessageText(LPCTSTR lpszText)
{
  SetWindowText(lpszText);
}
#endif


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
		
  if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
		!SetToolBar())
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

  if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
#ifdef WIN32
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

   ShowControlBar(&m_wndToolBar, AfxGetConfig()->m_bUIToolbar , FALSE);
   ShowControlBar(&m_wndStatusBar, AfxGetConfig()->m_bUIStatusbar , FALSE);
#endif

   SetMessage(IDS_MSG1_NOT_CHECKED,3);

#ifdef WIN95
   TrayMessage(GetSafeHwnd(), NIM_ADD, IDC_NOTIFY, IDI_SETA_EMPTY,strTray.GetBuffer(0));
#endif   

#ifndef FREEWARE
 if (!popup.LoadMenu(IDR_POPUP_MENU))
   AfxThrowResourceException();	
#else
 if (!popup.LoadMenu(IDR_POPUP_MENU_FW))
   AfxThrowResourceException();	
#endif

	SetAutoTransfer(AfxGetConfig()->m_bCHECKAutoNotify);
  
	TickCount = 0;
	SetTimer(ID_STARTUP_TIMER,50,NULL);
    
GetSystemMenu(FALSE)->AppendMenu(MF_SEPARATOR);
GetSystemMenu(FALSE)->AppendMenu(MF_POPUP,(UINT)popup.GetSubMenu(0)->m_hMenu,"e-Mail Notify Menu");

return 0;
}



void CMainFrame::ReStartUpdateTimer()
{
 if (AfxGetConfig()->m_bCHECKAutoNotify)
  {
   TickCount = GetTickCount();
   UINT BaseTime = AfxGetConfig()->m_uCHECKTime;
   if (BaseTime>600) 
    BaseTime = 60000;
   else if (BaseTime<10)
    BaseTime = 1000;
   else
    BaseTime = BaseTime*100;
   SetTimer(ID_UPDATE_MAIL_TIMER,BaseTime,NULL);
  }
 else
  TickCount = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG




/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnMailCheckmail() 
{
 if (UpdateState==ST_IDLE)
  {
   KillTimer(ID_UPDATE_MAIL_TIMER);
   SetTransfering(TRUE);
   UpdateState=ST_DIALUP;

   m_wndToolBar.SendMessage(WM_IDLEUPDATECMDUI,TRUE);
   SetMessage(IDS_UPDATE_MAIL_STARTED,1);

#if defined(WIN95) || defined(WINNT35)
   if ((!AfxGetConfig()->m_bRAMonitor)||DialUpIsConnected())
   {
    TRACE("Already connected or non Dial Up mode\n");
	bDialUp = FALSE;
    UpdateState = ST_DNS;
	SendAsyncDNS();
   }
   else
   {
    if (!AfxGetConfig()->m_bRADialIn)
	 {
	  TRACE("Do not want to dial in\n");

	  SetMessage(IDS_DIALUP_NODIALUP,3);
	  MailCheckEnd2();
	 }
	else if (DialUpConnect()!=0)
     {
	  TRACE("Dial-Up connection impossible\n");
	  MailCheckEnd1(DD_ERROR);
      SetMessage(IDS_ERROR_DIALUP_CONNECT,2);
	  MailCheckEnd2();
	 }
    else
     {
      bDialUp = TRUE;
      UpdateState = ST_DIALUP;
	  TRACE("Dial-In in progress\n");
      SetMessage(IDS_DIALUP_INPROGRESS,3);
 	 }
    }
#else
   bDialUp = FALSE;
   UpdateState = ST_DNS;
   SendAsyncDNS();
#endif
  }
}  
 

void CMainFrame::SendAsyncDNS()   
{
   if (m_bAsyncOk==TRUE)
   {
	SetMessage(IDS_MSG3_DNS_OK,3);
    OnUpdateMail(0,0);
   }
   else
   {	
   hAsyncDNS = popclient->SendAsyncDNS(GetSafeHwnd());
   if (hAsyncDNS==0)
		{
		 UpdateState = ST_IDLE;
	     MailCheckEnd1(DD_ERROR);
		 SetMessage(IDS_ERROR_DNS_LAUNCH,2);
	 	 MailCheckEnd2();
		}	
	 else
 		{
	     SetMessage(IDS_MSG3_DNS_LAUNCHED,3);
 		}
   }
}




afx_msg LONG CMainFrame::OnAsyncDNS(UINT ,LONG lparam)
{
 if (WSAGETASYNCERROR(lparam)==0)
  {
 	DEBUG5(("Async DNS Ok"));
    m_bAsyncOk = TRUE;
	SetMessage(IDS_MSG3_DNS_OK,3);
    OnUpdateMail(0,0);
  }
 else
  {
	DEBUG5(("Async DNS Failed"));
    m_bAsyncOk = FALSE;
    MailCheckEnd1(DD_ERROR);
    SetMessage(IDS_ERROR_DNS_FAILED,2);
    MailCheckEnd2();
	ErrorState = 1;
  }
 return 1;
}


afx_msg LONG CMainFrame::OnMailDownload(UINT wparam,LONG lparam)
{
 AfxGetEmailDoc()->UpdateAllViews(NULL);
 return 0;
}




afx_msg LONG CMainFrame::OnUpdateMail(UINT,LONG)
{
  UpdateState = ST_MAIL;
  SetMessage(IDS_UPDATE_MAIL_STARTED,3);
  CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
  emaildoc->GetMessages();
  return 0;
}


afx_msg LONG CMainFrame::OnUpdateThreadFinished(UINT wparam,LONG) 
{
  CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
  unsigned messages = emaildoc->GetMailNb();
  if (UpdateState==ST_CANCEL)
   {
    ErrorState = 2;
    MailCheckEnd1(DD_CANCEL);
    SetMessage(IDS_UPDATE_MAIL_CANCELLED,2);
   }
  else
   {
    ErrorState = 3;
    MailCheckEnd1(DD_NORMAL);
    if (wparam!=-1) ShowMailNumber(messages);
   }

#ifndef LISTVIEW
  emaildoc->ChangeSelectionTo(0);
#endif
  
  emaildoc->UpdateAllViews(NULL);
  MailCheckEnd2();
  m_wndToolBar.SendMessage(WM_IDLEUPDATECMDUI,TRUE);
  return 0;
}


void CMainFrame::OnTimer(UINT nIDEvent) 
{
  CFrameWnd::OnTimer(nIDEvent);

  if (nIDEvent==ID_UPDATE_MAIL_TIMER)
	{
	  if (TickCount!=0)
	  {
  	  if (AfxGetConfig()->m_bCHECKAutoNotify)
	   { 
	    TRACE("Current Tick Count: %lu Start Tick Count: %lu Timer: %lu Diff: %lu\n",
	           GetTickCount(),TickCount,AfxGetConfig()->m_uCHECKTime*1000,
	    	   ((GetTickCount()-TickCount)-(AfxGetConfig()->m_uCHECKTime*(DWORD)1000)));	
	    
	    
	    DWORD Time = 0;
#if defined(WIN95) || defined(WINNT35)
		if ((!AfxGetConfig()->m_bRAMonitor)||DialUpIsConnected())
		{
		 Time = (AfxGetConfig()->m_uCHECKTime*(DWORD)1000);
		}
		else
		{
		 Time = (AfxGetConfig()->m_uRATime*60L*1000L);	
		}
#else
		 Time = (AfxGetConfig()->m_uCHECKTime*(DWORD)1000);
#endif
	    if ((GetTickCount()-TickCount)>Time)
	     {
		  TickCount = 0;
	
#ifndef FREEWARE
#if defined(WIN95) || defined(WINNT35)
		  if (AfxGetConfig()->m_bCrippled==FALSE)
#endif
#endif
			  OnMailCheckmail();
  	     }

  	   }
     }
  	}
 
  else if (nIDEvent==ID_ANIMATE_TIMER)
   {
	if (UpdateState!=ST_IDLE)
	 {
	  bAnimate = !bAnimate;
	  if (bAnimate==TRUE)
	   {
		 UpdateTray(IDI_SETA_TRANSFER);
	   }
	  else
	   {
		 UpdateTray(IDI_SETA_EMPTY);
	   }
	 }
   }
  else if (nIDEvent==ID_STARTUP_TIMER)
   {
	KillTimer(ID_STARTUP_TIMER);
    TickCount = 0;
	if (AfxGetConfig()->m_bCHECKOnStartup)
	    OnMailCheckmail();
	  else
	    ReStartUpdateTimer();
   }
  else if (nIDEvent==ID_SAFEGUARD)
   {
	if (iSafeGuard!=-1)
	{
	iSafeGuard++;
	if (iSafeGuard>100)
	 {
	  OnStopUpdate();
	 }
	}

   }
}
					

void CMainFrame::OnStopUpdate() 
{
 if (UpdateState==ST_DIALUP)
  {
   ErrorState = 1;
   MailCheckEnd1(DD_CANCEL);
   MailCheckEnd2();
 }
 else if (UpdateState==ST_DNS)
  {
    MailCheckEnd1(DD_CANCEL);
 	if (m_bAsyncOk==FALSE)
	{
	 if (WSACancelAsyncRequest(hAsyncDNS)==0)
  		{
		 DEBUG5(("Async DNS cancelled succesfully"));
         SetMessage(IDS_MSG2_CANCELLED,2);
  		}
 	 else
  		{
		 DEBUG5(("Async DNS could not be cancelled"));
         SetMessage(IDS_MSG3_DNS_NOT_CANCELLED,2);
  		}
  	}
	ErrorState = 1;
    MailCheckEnd2();
  }
 else if (UpdateState==ST_MAIL)
  {  		
   UpdateState = ST_CANCEL;
   DEBUG5(("Cancelling transfer"));
   MailCheckEnd1(DD_CANCEL);
   SetMessage(IDS_MSG3_CANCELLING_TRANSFER,3);
   MailCheckEnd2();
   KillTimer(ID_ANIMATE_TIMER);
   popclient->StopUpdate();
  }
}



void CMainFrame::OnClearMailWindow() 
{
  if (UpdateState==ST_IDLE)
  {
   CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());

   emaildoc->OnNewDocument();

   // Set Title to "Mail not checked"
   SetMessage(IDS_MSG3_MAILBOX_CLEARED,3);
   UpdateTray(IDI_SETA_EMPTY);
  }
}




afx_msg LONG CMainFrame::OnNotifyIcon(UINT wparam,LONG lparam) 
{
#ifdef WIN95
 switch (lparam)
	{
	  case WM_LBUTTONUP:
		 ActivateFrame(SW_RESTORE);
	     SetForegroundWindow();
		 UpdateTitleBar();
		break;
	  case WM_RBUTTONDOWN:
		 {				
		    POINT pCursor;
		 	
		 	// Getting the Menu to appear correctly
			SetForegroundWindow();
			GetCursorPos(&pCursor);
		 										
			CMenu* pMenu = popup.GetSubMenu(0);
		     pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_RIGHTBUTTON,
		 	 pCursor.x,pCursor.y,this,NULL);

			// Getting the Menu to close if you do not choose anything
  		    PostMessage(WM_USER, 0, 0);
		 }
	  	break;
/*	  case WM_MOUSEMOVE:
       sndPlaySound(AfxGetConfig()->m_csNOTIFYWave,SND_ASYNC);	
	break;
*/	}	 
#endif
 return 1;
}


afx_msg LONG CMainFrame::OnDrawItem(UINT wparam,LONG lparam) 
{
#ifdef WIN95
  return (IconDrawItem((LPDRAWITEMSTRUCT)lparam));
#else
  return 0;
#endif
}



afx_msg void CMainFrame::OnSysCommand(UINT nID,LPARAM lparam) 
{
 switch(nID)
  {
   case ID_MAIL_READER:
   case ID_MAIL_CHECKMAIL:
   case ID_MAIL_OPTIONS:
   case ID_STOP_UPDATE:
   case ID_HELP_INDEX:
   case ID_APP_ABOUT:
   case ID_AUTO_TRANSFER:
   case ID_APP_EXIT:
	 SendMessage(WM_COMMAND,nID,0);
	break;
   default:
    CWnd::OnSysCommand(nID,lparam);
  }
}

BOOL CMainFrame::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
  LoadAccelTable(MAKEINTRESOURCE(IDR_MAINFRAME));	

  return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CMainFrame::OnClose() 
{
#ifndef WIN95
 MSG Msg;
 if ((UpdateState==ST_CANCEL)||(UpdateState!=ST_IDLE))
  {
   if (NotifyMessageBox(IDS_UPDATING_MAIL,MB_YESNO | MB_ICONQUESTION)==IDYES)
   {
    OnStopUpdate();
    while (UpdateState==ST_CANCEL)
      {
       if (::PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
       {
        ::TranslateMessage(&Msg);
        ::DispatchMessage(&Msg);
       }
	  }
	 CFrameWnd::OnClose();
   }
  }
 else
  {
   CFrameWnd::OnClose();
  }
#else
   if (bWin4) ShowWindow(SW_HIDE);
#endif
}


void CMainFrame::OnDestroy() 
{
 OnStopUpdate();
#ifdef LISTVIEW
 ((CEmailListView*)GetActiveView())->SaveColumnPreferences();
#endif
 
#ifdef WIN95
 TrayMessage(GetSafeHwnd(), NIM_DELETE, IDC_NOTIFY,NULL,NULL);
#endif

 AfxGetConfig()->SaveProfileString();
 CFrameWnd::OnDestroy();
}


	
void CMainFrame::OnShowEmail() 
{
 #ifdef WIN95
 if (bWin4)
  {
   if (IsWindowVisible())
    {
     ShowWindow(SW_HIDE);
    }
   else
    {
     ShowWindow(SW_RESTORE);
     UpdateTitleBar();
    }
  }
 else
#endif
 {
  if (IsIconic())
    ShowWindow(SW_RESTORE);
  else
    ShowWindow(SW_MINIMIZE);
 }
}


void CMainFrame::OnAutoTransfer() 
{
 SetAutoTransfer(!(AfxGetConfig()->m_bCHECKAutoNotify));
}


void CMainFrame::SetAutoTransfer(BOOL autotransfer)
{
 if (autotransfer)
   {
   popup.ModifyMenu(ID_AUTO_TRANSFER,MF_BYCOMMAND | MF_STRING,ID_AUTO_TRANSFER,"&Disable Notify");
   GetMenu()->ModifyMenu(ID_AUTO_TRANSFER,MF_BYCOMMAND | MF_STRING,ID_AUTO_TRANSFER,"&Disable Notify");	
   m_wndToolBar.SetButtonInfo(11,ID_AUTO_TRANSFER,TBBS_BUTTON,6);
   AfxGetConfig()->m_bCHECKAutoNotify = TRUE;
   ReStartUpdateTimer();
  }
 else
   {
   popup.ModifyMenu(ID_AUTO_TRANSFER,MF_BYCOMMAND | MF_STRING,ID_AUTO_TRANSFER,"&Enable Notify");
   GetMenu()->ModifyMenu(ID_AUTO_TRANSFER,MF_BYCOMMAND | MF_STRING,ID_AUTO_TRANSFER,"&Enable Notify");	
   m_wndToolBar.SetButtonInfo(11,ID_AUTO_TRANSFER,TBBS_BUTTON,5);
   AfxGetConfig()->m_bCHECKAutoNotify = FALSE;
   KillTimer(ID_UPDATE_MAIL_TIMER);
   }
}


void CMainFrame::SetTransfering(BOOL transfering)
{
 if (transfering)
   {
   SetTimer(ID_SAFEGUARD,5000,NULL);
   iSafeGuard = 0;
   popup.ModifyMenu(ID_MAIL_CHECKMAIL,MF_BYCOMMAND | MF_STRING,ID_STOP_UPDATE,"&Stop Mail Transfer");
   GetMenu()->ModifyMenu(ID_MAIL_CHECKMAIL,MF_BYCOMMAND | MF_STRING,ID_STOP_UPDATE,"&Stop Mail Transfer");	
   }
 else
   {
   KillTimer(ID_SAFEGUARD);
   iSafeGuard = -1;
   popup.ModifyMenu(ID_STOP_UPDATE,MF_BYCOMMAND | MF_STRING,ID_MAIL_CHECKMAIL,"&Check Mail");
   GetMenu()->ModifyMenu(ID_STOP_UPDATE,MF_BYCOMMAND | MF_STRING,ID_MAIL_CHECKMAIL,"&Check Mail");	
   }
}




#ifdef _DEBUG
void CMainFrame::MsgBox(const char* format,...)
{
 char message[255];
 va_list ap;

 va_start(ap,format);
 vsprintf(message, format, ap);
 va_end(ap);

 AfxMessageBox(message);
}


void CMainFrame::MsgBox(UINT MessageCode,...)
{
 char message[255];
 va_list ap;

 CString strmessage;
 if (strmessage.LoadString(MessageCode))
  {
   va_start(ap,strmessage);
   vsprintf(message, strmessage, ap);
   va_end(ap);

   AfxMessageBox(message);
  }
 else
  {
   AfxMessageBox("Internal Error");
  } 
}
#endif



void CMainFrame::UpdateTray(UINT IconCode) 
{
 IdIcon = IconCode;
 if ((!bWin4)&&IsIconic())
  {
	Invalidate(TRUE);
  }

 CString strMsg = AfxGetConfig()->m_csPOPServer;
 strMsg += "(";
 strMsg += AfxGetConfig()->m_csPOPUsername;
 strMsg += "): ";
 strMsg += strTray; 


#ifdef WIN95
   TrayMessage(GetSafeHwnd(), NIM_MODIFY, IDC_NOTIFY, IconCode,strMsg.GetBuffer(0));
#endif   
}

afx_msg LONG CMainFrame::OnSetMessage(UINT wParam,LONG lParam) 
{
 SetMessage(wParam,lParam);
 return 1;
}



void CMainFrame::SetMessage(UINT MessageCode,UINT Type,int param1)
{
 char message[255];
 CString strmessage;
 
 if (!strmessage.LoadString(MessageCode))
   AfxThrowResourceException();
 
 const char* msg = strmessage.GetBuffer(0);
 sprintf(message,msg,param1);
 SetMessage(message,Type);
}


void CMainFrame::SetMessage(UINT MessageCode,UINT Type,int param1,int param2)
{
 char message[255];
 CString strmessage;
 
 if (!strmessage.LoadString(MessageCode))
   AfxThrowResourceException();
 
 const char* msg = strmessage.GetBuffer(0);
 sprintf(message,msg,param1,param2);
 SetMessage(message,Type);
}



void CMainFrame::SetMessage(UINT MessageCode,UINT Type)
{
 CString strmessage;
 if (!strmessage.LoadString(MessageCode))
   AfxThrowResourceException();
 
 const char* msg = strmessage.GetBuffer(0);
 SetMessage(msg,Type);
}


//
// Do not call this function from a different Thread than this one
//
void CMainFrame::SetMessage(const char* message,UINT Type)
{
 switch (Type)
  {
   case 1:
 	strTitle = message;
 	strTray = message;
	strStatus = message;
    GetActiveDocument()->SetTitle(message);

    SetMessageText(message);

    GetActiveDocument()->SetTitle(message);
	break;

  case 2:
 	strTray = message;
	strStatus = message;
 
    SetMessageText(message);

	if (AfxGetConfig()->m_bUIMsgBoxOnError)
	 NotifyMessageBox(message);
	UpdateTray(IDI_SETA_ERROR);
	SetMessage(IDS_MSG1_NOT_CHECKED,5);
	break;

  case 3:
 	strTray = message;
	strStatus = message;

    SetMessageText(message);
	break;

  case 4:
 	strTitle = message;
 	strTray = message;
	strStatus = message;
   break;

  case 5:
	strTitle = message;
    GetActiveDocument()->SetTitle(message);
   break;
  }
}



void CMainFrame::ShowMailNumber(unsigned nbmail)
{
 CString string;
 BOOL ret;
 char Title[100];
 
 if (nbmail==0)
  ret = string.LoadString(IDS_MSG1_NOMAILINBOX);
 else
  ret = string.LoadString(IDS_MSG1_MAILINBOX);

 if (ret)
  {
   sprintf(Title,string,nbmail); 
  }
 else
  {
   AfxThrowResourceException();
  }

 SetMessage(Title,1);
 if (nbmail>9) nbmail = 9;
 UpdateTray(IDI_SETA_EMPTY+nbmail);
}



void CMainFrame::UpdateTitleBar()
{
 ((CEmailDoc*)GetActiveDocument())->SetTitle(strTitle);

  SetMessageText(strStatus);
}

void CMainFrame::OnMailOptions2() 
{
 if (UpdateState==ST_IDLE)
  {
   CString csOldServer = AfxGetConfig()->m_csPOPServer;

   KillTimer(ID_UPDATE_MAIL_TIMER);
   CPSOptions psOptions(IDS_OPTIONS_CAPTION,this,0);
   
   psOptions.DoModal();
   if (AfxGetConfig()->m_csPOPServer != csOldServer)
    m_bAsyncOk = FALSE;
   SetAutoTransfer(AfxGetConfig()->m_bCHECKAutoNotify);
  }
}

void CMainFrame::OnMailReader() 
{
 if (WinExec(AfxGetConfig()->m_csNOTIFYMailReader,SW_SHOW)<=31)
	NotifyMessageBox(IDS_MAIL_READER_ERROR);
 else
	OnClearMailWindow();
}


void CMainFrame::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
 CFrameWnd::OnLButtonDblClk(nFlags, point);
}

void CMainFrame::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
    if ((!bWin4)&&IsIconic())
	 {
	   DefWindowProc(WM_ICONERASEBKGND, (WORD)dc.m_hDC, 0L);
        // Center the icon
        CRect rc;
        GetClientRect(&rc);
        rc.left = (rc.right  - ::GetSystemMetrics(SM_CXICON)) >> 1;
        rc.top  = (rc.bottom - ::GetSystemMetrics(SM_CYICON)) >> 1;
        // Draw the icon
        HICON hIcon = AfxGetApp()->LoadIcon(IdIcon);
        dc.DrawIcon(rc.left, rc.top, hIcon);
	 }
	// Do not call CFrameWnd::OnPaint() for painting messages
}


BOOL CMainFrame::OnEraseBkgnd(CDC* pDC)
  {
     if (IsIconic())
        return TRUE;
     else
        return CFrameWnd::OnEraseBkgnd(pDC);
  }


HCURSOR CMainFrame::OnQueryDragIcon()
  {
     return AfxGetApp()->LoadIcon(IdIcon);
  }


BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs) 
{
 if (bWin4)
  return CFrameWnd::PreCreateWindow(cs);
 else
  {
    if (CFrameWnd::PreCreateWindow(cs))
     {

		cs.lpszClass = AfxRegisterWndClass ( 
                           CS_DBLCLKS | CS_HREDRAW |
                           CS_VREDRAW,
                           AfxGetApp()->LoadStandardCursor(IDC_ARROW),
                                           (HBRUSH) (COLOR_WINDOW + 1),
                                            0); 
      return TRUE;
     }    
    else 
      return NULL;
  }
}
	

void CMainFrame::OnAppExit() 
{
 MSG Msg;
 if ((UpdateState==ST_CANCEL)||(UpdateState!=ST_IDLE))
  {
   if (NotifyMessageBox(IDS_UPDATING_MAIL,MB_YESNO | MB_ICONQUESTION)==IDYES)
   {
    OnStopUpdate();
    while (UpdateState==ST_CANCEL)
      {
	  if (::PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
       {
        ::TranslateMessage(&Msg);
        ::DispatchMessage(&Msg);
       }
	  }
	 CFrameWnd::OnClose();
   }
  }
 else
  {
   CFrameWnd::OnClose();
  }
}

void CMainFrame::OnHelpIndex() 
{
 #if defined(WIN95) || defined(WIN32S)
  AfxGetApp()->WinHelp(0,HELP_FINDER);	
 #else
  AfxGetApp()->WinHelp(0,HELP_CONTENTS);	
 #endif 
}


void CMainFrame::OnHelp() 
{
 #if defined(WIN95) || defined(WIN32S)
  AfxGetApp()->WinHelp(0,HELP_FINDER);	
 #else
  AfxGetApp()->WinHelp(0,HELP_CONTENTS);	
 #endif 
}


void CMainFrame::OnMailDelete() 
{
#ifndef FREEWARE
#if defined(WIN95) || defined(WINNT35)
if (AfxGetConfig()->m_bCrippled==FALSE)
#endif
#endif
 {
  if (UpdateState==ST_IDLE)
   {
    KillTimer(ID_UPDATE_MAIL_TIMER);
    SetTransfering(TRUE);
    UpdateState = ST_MAIL;

	if (AfxGetConfig()->m_bPOPPasswordAskAlways)
	 CPasswordDlg::CheckPassword();

    SetMessage(IDS_UPDATE_MAIL_STARTED,3);
    CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
    emaildoc->MailDelete();
   }
  }
#ifndef FREEWARE
 #if defined(WIN95) || defined(WINNT35)
  else
  {
   AfxMessageBox(IDS_EVALUATION);
  }
 #endif
#endif

}


void CMainFrame::OnMailView() 
{

#ifndef FREEWARE
#if defined(WIN95) || defined(WINNT35)
if (AfxGetConfig()->m_bCrippled==FALSE)
#endif
#endif
  {
  if (UpdateState==ST_IDLE)
   {
    KillTimer(ID_UPDATE_MAIL_TIMER);
    SetTransfering(TRUE);
    UpdateState = ST_MAIL;
   
    if (AfxGetConfig()->m_bPOPPasswordAskAlways)
	 CPasswordDlg::CheckPassword();
	
	SetMessage(IDS_UPDATE_MAIL_STARTED,3);
    CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
    emaildoc->MailView();
   }
  }
#ifndef FREEWARE
 #if defined(WIN95) || defined(WINNT35)
  else
  {
   AfxMessageBox(IDS_EVALUATION);
  }
 #endif
#endif
}

afx_msg LONG CMainFrame::OnDeleteThreadFinished(UINT wparam,LONG) 
{
  MailCheckEnd1(DD_NORMAL);
  CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
  unsigned messages = emaildoc->GetMailNb();
  
  ShowMailNumber(messages);
  
  if ((!AfxGetPOPClient()->bCanceling)&&(wparam==FALSE))
   NotifyMessageBox(IDS_DELETE_MAIL_FAILED);
  
   emaildoc->UpdateAllViews(NULL);
   MailCheckEnd2();
   m_wndToolBar.SendMessage(WM_IDLEUPDATECMDUI,TRUE);
  return 0;
}


afx_msg LONG CMainFrame::OnViewThreadFinished(UINT wparam,LONG) 
{
  MailCheckEnd1(DD_NORMAL);
  MailCheckEnd2();
  CEmailDoc* emaildoc = ((CEmailDoc*)GetActiveDocument());
  ShowMailNumber(emaildoc->GetMailNb()); 
  m_wndToolBar.SendMessage(WM_IDLEUPDATECMDUI,TRUE);
  return 0;
}


afx_msg LONG CMainFrame::OnNotifyProgress(UINT MessageCode,LONG Progress)
{
 CString strmessage;
 
 if (!strmessage.LoadString(MessageCode))
   AfxThrowResourceException();

 char sprogress[256];
 sprintf(sprogress,"%s..%d%%",strmessage.GetBuffer(0),(int)Progress);
 
 SetMessage(sprogress,3);  
 return 0;
}



void CMainFrame::OnUpdateMailCheckmail(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(UpdateState==ST_IDLE);
}

void CMainFrame::OnUpdateMailOptions(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(UpdateState==ST_IDLE);
}

void CMainFrame::OnUpdateStopUpdate(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(UpdateState!=ST_IDLE);	
}

void CMainFrame::OnUpdateClearMailWindow(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable((UpdateState==ST_IDLE));	
}


void CMainFrame::OnUpdateAppAbout(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(TRUE); 	
}


void CMainFrame::OnUpdateMailFunction(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((UpdateState==ST_IDLE)&&(((CEmailDoc*)GetActiveDocument())->GetMailSelected()>0));	
}



DWORD CMainFrame::DialUpConnect()
{
#if defined(WIN95) || defined(WINNT35)
  RASDIALPARAMS rdpParams;
  rdpParams.dwSize = sizeof(RASDIALPARAMS);
  lstrcpy(rdpParams.szEntryName,AfxGetConfig()->m_csRAService);
  lstrcpy(rdpParams.szPhoneNumber,"");
  lstrcpy(rdpParams.szCallbackNumber,"*");
  lstrcpy(rdpParams.szUserName,AfxGetConfig()->m_csRALogin);
  lstrcpy(rdpParams.szPassword,AfxGetConfig()->m_csRAPassword);
  lstrcpy(rdpParams.szDomain,"*");

  uMsg = RegisterWindowMessageA( RASDIALEVENT ); 
  if (uMsg == 0) 
	uMsg = WM_RASDIALEVENT;    

  return (*lpfnRasDial)(NULL,NULL,&rdpParams,0xFFFFFFFF,GetSafeHwnd(),&hrcRASHandle);
#endif
 return 1;
}


DWORD CMainFrame::DialUpDisconnect(UINT uReason,UINT msgnb)
{
#if defined(WIN95) || defined(WINNT35)
 if (bRasOk&&bDialUp&&hrcRASHandle)
  {
   if (AfxGetConfig()->m_bRAAskDisconnect)
    {
	 CString strMsg;	   
	
	 switch (uReason)
	 {
		 case DD_CANCEL:
			 AfxFormatString1(strMsg,IDS_DIALUPDISCONNECT_CANCEL,"");
			 break;
		 case DD_ERROR:
			 AfxFormatString1(strMsg,IDS_DIALUPDISCONNECT_ERROR,"");
			 break;
		 case DD_NORMAL:
			 char cNb[10];
 			 sprintf(cNb,"%d",msgnb);
              AfxFormatString1(strMsg,IDS_DIALUPDISCONNECT_NORMAL,cNb);
			 break;
	 }
		 
	if (NotifyMessageBox(strMsg,MB_YESNO)==IDNO)
  	{
	 bDialUp = FALSE;
	 return 0;
	}
   }
	   
	DWORD dwReturn = (*lpfnRasHangUp)(hrcRASHandle);
   hrcRASHandle = 0;
   return dwReturn;
  }
 else
#endif
  return 0;
}


BOOL CMainFrame::DialUpIsConnected()
{
#if defined(WIN95) || defined(WINNT35)
 if (bRasOk)
 {
   RASCONN rcRasConnection[5];
   DWORD dwSize = 5*sizeof(RASCONN);
   DWORD dwNbEntries;
   rcRasConnection[0].dwSize = sizeof(RASCONN);

   if ((*lpfnRasEnumConnections)(rcRasConnection,&dwSize,&dwNbEntries)==0)
    {
	 if (dwNbEntries!=0)
	  return TRUE;
    };
   return FALSE;
  }
  else
 #endif
  return TRUE;
}

#if defined(WIN95) || defined(WINNT35)
afx_msg LONG CMainFrame::OnRasDialEvent(UINT rasconnstate,LONG dwError)
{
 if ((dwError!=0)||(rasconnstate&RASCS_PAUSED))
  {
   (*lpfnRasHangUp)(hrcRASHandle);
   hrcRASHandle = 0;
   
   MailCheckEnd1(DD_ERROR);
   
   char buffer[256];
   if ((*lpfnRasGetErrorString)(dwError,buffer,256))
    NotifyMessageBox(buffer);
   
   AfxGetMainFrame()->SendMessage(WM_SET_MESSAGE,IDS_ERROR_DIALUP_CONNECT,2);
   
   MailCheckEnd2();
  }
 else if (rasconnstate==RASCS_DONE)
  {
   bDialUp = TRUE;
   UpdateState = ST_DNS;
   Sleep(1000);
   SendAsyncDNS();
  }
  return 0;
 }
#endif




void CMainFrame::MailCheckEnd1(UINT uReason,UINT msgnb)
{
 KillTimer(ID_SAFEGUARD);
 iSafeGuard = -1;
 DialUpDisconnect(uReason);
}


void CMainFrame::MailCheckEnd2()
{
 ReStartUpdateTimer();
 UpdateState = ST_IDLE;
 SetTransfering(FALSE);
}


#ifdef NEWTOOLTIP
void CMainFrame::OnTestTooltip() 
{
 if (notifystatus==NULL)
 {
  notifystatus = new CNotifyStatus();
  notifystatus->CreateStatusWindow();
  notifystatus->ShowStatusWindow();
 }
 else
 {
  notifystatus->ShowStatusWindow();
 }
}
#endif

void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
if (!IsIconic())
 { 
    CRect crRect;
	GetWindowRect(&crRect);

	char sbuffer[100];
	sprintf(sbuffer,"%d,%d,%d,%d",crRect.left,crRect.top,
							crRect.right,crRect.bottom);

	AfxGetConfig()->m_csUIMainWindowPos = sbuffer;
  }
}

void CMainFrame::OnMove(int x, int y) 
{
	CFrameWnd::OnMove(x, y);
	
 if (!IsIconic())
 { 
	CRect crRect;

	GetWindowRect(&crRect);

	char sbuffer[100];
	sprintf(sbuffer,"%d,%d,%d,%d",crRect.left,crRect.top,
							crRect.right,crRect.bottom);

	AfxGetConfig()->m_csUIMainWindowPos = sbuffer;
 }
}

#ifndef WIN16
void CMainFrame::OnViewStatusBar() 
{
 BOOL bShow = (m_wndStatusBar.GetStyle() & WS_VISIBLE) == 0;
 AfxGetConfig()->m_bUIStatusbar = bShow;
 ShowControlBar(&m_wndStatusBar, bShow , FALSE);	
}

void CMainFrame::OnViewToolbar() 
{
 BOOL bShow = (m_wndToolBar.GetStyle() & WS_VISIBLE) == 0;
 AfxGetConfig()->m_bUIToolbar = bShow;
 ShowControlBar(&m_wndToolBar, bShow , FALSE);	
}
#endif   


LRESULT CMainFrame::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
#if defined(WIN95) || defined(WINNT35)
	if (message==uMsg)
     return CMainFrame::OnRasDialEvent(wParam,lParam);
#endif	
	return CFrameWnd::WindowProc(message, wParam, lParam);
}


/*
static UINT BASED_CODE buttons[] =
{
	ID_CLEAR_MAIL_WINDOW,
		ID_SEPARATOR,
	ID_MAIL_CHECKMAIL,
		ID_SEPARATOR,
	ID_MAIL_DELETE,
	ID_MAIL_VIEW
		ID_SEPARATOR,
	ID_MAIL_OPTIONS,
		ID_SEPARATOR,
	ID_SHOW_EMAIL,
	ID_AUTO_TRANSFER,
 		ID_SEPARATOR,
	ID_APP_ABOUT
};
*/

BOOL CMainFrame::SetToolBar()
{
 if (m_wndToolBar.SetButtons(NULL,14)==FALSE)
  return FALSE; 
 m_wndToolBar.SetButtonInfo(0,ID_CLEAR_MAIL_WINDOW,TBBS_BUTTON,0);
 m_wndToolBar.SetButtonInfo(1,0,TBBS_SEPARATOR,4);
 m_wndToolBar.SetButtonInfo(2,ID_MAIL_CHECKMAIL,TBBS_BUTTON,1);
 m_wndToolBar.SetButtonInfo(3,ID_STOP_UPDATE,TBBS_BUTTON,2);
 m_wndToolBar.SetButtonInfo(4,0,TBBS_SEPARATOR,4);
 m_wndToolBar.SetButtonInfo(5,ID_MAIL_DELETE,TBBS_BUTTON,9);
 m_wndToolBar.SetButtonInfo(6,ID_MAIL_VIEW,TBBS_BUTTON,10);
 m_wndToolBar.SetButtonInfo(7,0,TBBS_SEPARATOR,4);
 m_wndToolBar.SetButtonInfo(8,ID_MAIL_OPTIONS,TBBS_BUTTON,3);
 m_wndToolBar.SetButtonInfo(9,0,TBBS_SEPARATOR,4);
 m_wndToolBar.SetButtonInfo(10,ID_SHOW_EMAIL,TBBS_BUTTON,4);
 m_wndToolBar.SetButtonInfo(11,ID_AUTO_TRANSFER,TBBS_BUTTON,5);
 m_wndToolBar.SetButtonInfo(12,0,TBBS_SEPARATOR,4);
 m_wndToolBar.SetButtonInfo(13,ID_APP_ABOUT,TBBS_BUTTON,7);
 return TRUE;
}
